package admin_record

import (
	"github.com/rdlucklib/rdluck_tools/orm"
	"time"
)

type AdminRecord struct {
	UserRecordId   int       `orm:"column(user_record_id);pk" json:"user_record_id"` // 用户id
	OpenId         string    `orm:"column(open_id);" json:"open_id"`                 // open_id
	UnionId        string    `orm:"column(union_id);" json:"union_id"`
	Subscribe      int       `orm:"column(subscribe);" json:"subscribe"`       // 是否关注
	NickName       string    `orm:"column(nick_name);" json:"nick_name"`       // 用户昵称
	RealName       string    `orm:"column(real_name);" json:"real_name"`       // 用户实际名称
	BindAccount    string    `orm:"column(bind_account);" json:"bind_account"` // 绑定时的账号
	Sex            int       `orm:"column(sex);" json:"sex"`                   // 普通用户性别，1为男性，2为女性
	Province       string    `orm:"column(province);" json:"province"`         // 普通用户个人资料填写的省份
	City           string    `orm:"column(city);" json:"city"`                 // 普通用户个人资料填写的城市
	Country        string    `orm:"column(country);" json:"country"`           // 国家，如中国为CN
	Headimgurl     string    `orm:"column(headimgurl);" json:"headimgurl"`
	CreateTime     time.Time `orm:"column(create_time);" json:"create_time"`         // 创建时间
	CreatePlatform int       `orm:"column(create_platform);" json:"create_platform"` // 注册平台,1:日度点评公众号,2:管理后台，3：pc端网站,4:查研观向小程序
	SessionKey     string    `orm:"column(session_key);" json:"session_key"`         // 微信小程序会话密钥
	UserId         int       `orm:"column(user_id);" json:"user_id"`                 // 用户id
}

//根据openid获取用户关系
func GetAdminRecordByOpenId(openId string) (item *AdminRecord, err error) {
	sql := `SELECT * FROM admin_record WHERE open_id=? `
	err = orm.NewOrm().Raw(sql, openId).QueryRow(&item)
	return
}

//根据用户id和平台id获取用户关系
func GetAdminRecordByAdminId(adminId, platform int) (item *AdminRecord, err error) {
	sql := `SELECT * FROM admin_record WHERE user_id=? AND create_platform = ?`
	err = orm.NewOrm().Raw(sql, adminId, platform).QueryRow(&item)
	return
}

//添加用户关系
func AddAdminRecord(record *AdminRecord) (recordId int64, err error) {
	o := orm.NewOrm()
	recordId, err = o.Insert(record)
	return
}

//根据openid绑定用户关系
func BindAdminRecordByOpenid(adminId int, openId string) (err error) {
	o := orm.NewOrm()
	msql := " UPDATE admin_record SET user_id=? WHERE open_id = ? "
	_, err = o.Raw(msql, adminId, openId).Exec()
	return
}

//根据openid解除绑定用户关系
func UnBindAdminRecordByOpenid(openId string) (err error) {
	o := orm.NewOrm()
	msql := ` UPDATE admin_record SET user_id = 0 WHERE open_id = ? `
	_, err = o.Raw(msql, openId).Exec()
	return
}

//修改用户微信信息
func ModifyAdminRecordInfo(openId, nickName, headimgUrl, city, province, country string, sex, adminId int) (err error) {
	o := orm.NewOrm()
	sql := `UPDATE admin_record SET nick_name=?,headimgurl=?,sex=?,city=?,province=?,country=? WHERE user_id=? and openid=? `
	_, err = o.Raw(sql, nickName, headimgUrl, sex, city, province, country, adminId, openId).Exec()
	return
}

//获取该用户第一个的 三方信息（微信头像信息）
func GetAdminThirdRecordByAdminId(adminId int) (item *AdminRecord, err error) {
	sql := `SELECT * FROM admin_record WHERE user_id = ? order by user_record_id asc`
	err = orm.NewOrm().Raw(sql, adminId).QueryRow(&item)
	return
}
