package approval_flow_node

import (
	"github.com/rdlucklib/rdluck_tools/orm"
	"time"
)

//审批流程节点路线表
type ApprovalFlowNode struct {
	NodeId     int       `orm:"column(node_id);pk" json:"node_id"`         // 流程节点id
	FlowId     int       `orm:"column(flow_id);" json:"flow_id"`           // 流程id
	NodeName   string    `orm:"column(node_name);" json:"node_name"`       // 流程节点名称
	PrevNodeId int       `orm:"column(prev_node_id);" json:"prev_node_id"` //上级节点id，为0代表是开始节点
	NextNodeId int       `orm:"column(next_node_id);" json:"next_node_id"` //下级节点id，为0代表是结束节点
	NodeType   string    `orm:"column(node_type);" json:"node_type"`       //节点类型，审批人：check；抄送人：cc（Carbon Copy），默认是：check
	AuditType  int       `orm:"column(audit_type);" json:"audit_type"`     //多人审批时，1：或签（一名审批人同意即可），2：会签（需所有审批人同意）
	User       string    `orm:"column(user);" json:"user"`                 //所属用户，多个用英文,隔开；用户类型为user代表指定人id，manager代表几级主管，role代表角色类型
	Version    int       `orm:"column(version);" json:"version"`           // 所属流程版本
	CreateTime time.Time `orm:"column(create_time);" json:"create_time"`   // 创建时间
}

//审批列表数据
type ApprovalFlowNodeList struct {
	NodeId     int       `description:"流程节点id"`                                       // 流程节点id
	FlowId     int       `description:"流程id"`                                         // 流程id
	NodeName   string    `description:"流程节点名称"`                                       // 流程节点名称
	PrevNodeId int       `description:"上级节点id，为0代表是开始节点"`                             //上级节点id，为0代表是开始节点
	NextNodeId int       `description:"下级节点id，为0代表是结束节点"`                             //下级节点id，为0代表是结束节点
	NodeType   string    `description:"节点类型，审批人：check；抄送人：cc（Carbon Copy），默认是：check"` //节点类型，审批人：check；抄送人：cc（Carbon Copy），默认是：check
	AuditType  int       `description:"多人审批时，1：或签（一名审批人同意即可），2：会签（需所有审批人同意）"`         //多人审批时，1：或签（一名审批人同意即可），2：会签（需所有审批人同意）
	User       string    `json:"-"`                                                   //所属用户，多个用英文,隔开；用户类型为user代表指定人id，manager代表几级主管，role代表角色类型
	Version    int       `description:"所属流程版本"`                                       // 所属流程版本
	CreateTime time.Time `description:"创建时间"`                                         // 创建时间
	UserList   []User    `description:"审批人、抄送人列表信息"`                                  // 审批人、抄送人列表信息
}

//审批人、抄送人信息
type User struct {
	AdminId      int    `description:"人员id（审批人、抄送人）"`
	Name         string `description:"人员名称（审批人、抄送人）"`
	RoleTypeCode string `description:"人员角色类型（审批人、抄送人）"`
	Mobile       string `description:"手机号（审批人、抄送人）"`
}

//根据流程id获取整个审批流程节点路线
func GetListByFlowId(flowId, version int) (items []*ApprovalFlowNodeList, err error) {
	sql := `SELECT * FROM approval_flow_node WHERE flow_id=? AND version=? order by node_id asc `
	o := orm.NewOrm()
	_, err = o.Raw(sql, flowId, version).QueryRows(&items)
	return
}

//根据节点id获取节点信息
func GetByNodeId(nodeId int) (item *ApprovalFlowNode, err error) {
	sql := `SELECT * FROM approval_flow_node WHERE node_id=? LIMIT 1 `
	o := orm.NewOrm()
	err = o.Raw(sql, nodeId).QueryRow(&item)
	return
}
